package com.sxtvs.api.chatgpt.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.api.chatgpt.dto.CompletionsParamsDto;
import com.sxtvs.api.chatgpt.dto.CompletionsRequestDto;
import com.sxtvs.api.chatgpt.dto.CompletionsResponseDto;
import com.sxtvs.api.chatgpt.dto.GptResponse;
import com.sxtvs.api.youmei.service.YoumeiAccountServiceImpl;
import com.sxtvs.core.advice.BizException;
import com.sxtvs.core.sls.AliyunLogger;
import lombok.Cleanup;
import lombok.SneakyThrows;
import org.apache.commons.io.IOUtils;
import org.apache.hc.core5.http.HttpStatus;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;

import java.nio.charset.StandardCharsets;
import java.util.concurrent.ArrayBlockingQueue;

@Service
public class ChatGptService {

    @Autowired
    private AliyunLogger logger;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private YoumeiAccountServiceImpl youmeiAccountService;

    private final CloseableHttpClient client = HttpClients.createDefault();


    private final ArrayBlockingQueue<String> queue = new ArrayBlockingQueue<>(1);

    {
        try {
//            queue.put("Bearer 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");
            queue.put("Bearer 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");
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    @SneakyThrows
    private synchronized String request(String params) {
        while (true) {
            String result;
            var httpPost = new HttpPost("https://api.openai.com/v1/completions");
            httpPost.setHeader("Authorization", "Bearer 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");
            httpPost.setEntity(new StringEntity(params, ContentType.APPLICATION_JSON));
            @Cleanup
            var response = client.execute(httpPost);
            @Cleanup
            var content = response.getEntity().getContent();
            result = IOUtils.toString(content, StandardCharsets.UTF_8);
            if (result.contains("The server had an error while processing your request")) {
                continue;
            }
            logger.info("response", result);
            if (response.getStatusLine().getStatusCode() >= HttpStatus.SC_REDIRECTION) {
                logger.error("error", result);
                throw new BizException(1000, "当前访问人数过多，请重试。");
            }
            return result;

        }

    }


    @SneakyThrows
    public CompletionsResponseDto completions(@RequestBody CompletionsRequestDto dto) {
        var text = dto.toText();
        var paramsDto = new CompletionsParamsDto(text);
        var params = objectMapper.writeValueAsString(paramsDto);
        logger.info("key", "completions",
                "request", params,
                "dto", objectMapper.writeValueAsString(dto),
                "userId", dto.getUserId(),
                "userName", dto.getUserName()
        );

        if (youmeiAccountService.wordCheckCache(text)) {
            throw new BizException(1001, "您的问题不符合相关法律法规，请修改后重试。");
        }

        var body = request(params);
        var gptResponse = objectMapper.readValue(body, GptResponse.class);
        var completionsResponseDto = new CompletionsResponseDto();
        var result = gptResponse.getChoices().stream().map(GptResponse.ChoicesDTO::getText).findFirst().orElse("").trim();
        completionsResponseDto.setResult(result);
        return completionsResponseDto;
    }
}
